# Django
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

# Third party
from cms.toolbar_base import CMSToolbar
from cms.toolbar_pool import toolbar_pool

# Project
from cookie_optin.conf.settings import ENABLE_OVERVIEW
from cookie_optin.utils import is_kapt_user


@toolbar_pool.register
class KlaroToolbar(CMSToolbar):
    """Adds a 'Statistiques' menu to the CMS toolbar that opens the consent overview modal.
    Uses a menu with position=-1 so it appears last (after Help and Language menus).
    Shown when COOKIE_OPTIN_ENABLE_OVERVIEW is True, or for Kapt users (with distinct label).
    """

    def populate(self):
        show_for_all = ENABLE_OVERVIEW
        show_for_kapt = not show_for_all and is_kapt_user(self.request.user)
        if not show_for_all and not show_for_kapt:
            return
        overview_url = reverse("cookie_optin:overview")
        visitor_url = reverse("cookie_optin:visitor_overview")
        menu_label = _("Statistiques (Kapt)") if show_for_kapt else _("Statistiques")
        klaro_menu = self.toolbar.get_or_create_menu("klaro", menu_label, position=-1)
        klaro_menu.add_modal_item(
            _("Suivi des consentements"), url=overview_url, on_close=None
        )
        klaro_menu.add_modal_item(_("Visiteurs"), url=visitor_url, on_close=None)



# Django
import django.db.models.deletion
from django.db import migrations, models

# Third party
import parler.models


class Migration(migrations.Migration):
    dependencies = [("sites", "0001_initial")]

    operations = [
        migrations.CreateModel(
            name="Application",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("slug", models.SlugField(max_length=100, verbose_name="Slug")),
                (
                    "is_required",
                    models.BooleanField(
                        default=False,
                        help_text="Only if the application is required for the site to function correctly.",
                        verbose_name="Is required",
                    ),
                ),
            ],
            options={
                "verbose_name": "Application",
                "verbose_name_plural": "Applications",
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="ApplicationTranslation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        max_length=15, verbose_name="Language", db_index=True
                    ),
                ),
                ("name", models.CharField(max_length=100, verbose_name="Name")),
                (
                    "description",
                    models.CharField(max_length=500, verbose_name="Description"),
                ),
                (
                    "master",
                    models.ForeignKey(
                        related_name="translations",
                        editable=False,
                        to="cookie_optin.Application",
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                    ),
                ),
            ],
            options={
                "managed": True,
                "db_table": "cookie_optin_application_translation",
                "db_tablespace": "",
                "default_permissions": (),
                "verbose_name": "Application Translation",
            },
        ),
        migrations.CreateModel(
            name="Cookie",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("name", models.CharField(max_length=250, verbose_name="Name")),
                (
                    "is_regexp",
                    models.BooleanField(
                        default=False, verbose_name="Regular expression"
                    ),
                ),
                (
                    "path",
                    models.CharField(
                        max_length=250, null=True, verbose_name="Path", blank=True
                    ),
                ),
                (
                    "application",
                    models.ForeignKey(
                        verbose_name="Application",
                        to="cookie_optin.Application",
                        on_delete=django.db.models.deletion.CASCADE,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        verbose_name="Site",
                        to="sites.Site",
                        on_delete=django.db.models.deletion.CASCADE,
                    ),
                ),
            ],
            options={"verbose_name": "Cookie", "verbose_name_plural": "Cookies"},
        ),
        migrations.CreateModel(
            name="Purpose",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                )
            ],
            options={"verbose_name": "Purpose", "verbose_name_plural": "Purposes"},
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="PurposeTranslation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        max_length=15, verbose_name="Language", db_index=True
                    ),
                ),
                ("name", models.CharField(max_length=100, verbose_name="Name")),
                (
                    "master",
                    models.ForeignKey(
                        related_name="translations",
                        editable=False,
                        to="cookie_optin.Purpose",
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                    ),
                ),
            ],
            options={
                "managed": True,
                "db_table": "cookie_optin_purpose_translation",
                "db_tablespace": "",
                "default_permissions": (),
                "verbose_name": "Purpose Translation",
            },
        ),
        migrations.AddField(
            model_name="application",
            name="purposes",
            field=models.ManyToManyField(
                to="cookie_optin.Purpose", verbose_name="Purposes"
            ),
        ),
        migrations.AlterUniqueTogether(
            name="purposetranslation",
            unique_together={("language_code", "master")},
        ),
        migrations.AlterUniqueTogether(
            name="applicationtranslation",
            unique_together={("language_code", "master")},
        ),
    ]



# Generated by Django 4.2.28

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cookie_optin", "0009_add_country_counts"),
    ]

    operations = [
        migrations.AddField(
            model_name="dailyconsentstats",
            name="source_counts",
            field=models.JSONField(
                blank=True,
                default=dict,
                verbose_name="Visitors by traffic source",
            ),
        ),
        migrations.AddField(
            model_name="dailyconsentstats",
            name="referral_counts",
            field=models.JSONField(
                blank=True,
                default=dict,
                verbose_name="Visitors by referral domain",
            ),
        ),
    ]



# Generated by Django 4.2.28 on 2026-03-03

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cookie_optin", "0006_dailyconsentstats"),
    ]

    operations = [
        migrations.AddField(
            model_name="dailyconsentstats",
            name="show_count",
            field=models.PositiveIntegerField(
                default=0,
                verbose_name="Banner displays (visitors)",
            ),
        ),
        migrations.AddField(
            model_name="dailyconsentstats",
            name="no_interaction_count",
            field=models.PositiveIntegerField(
                default=0,
                verbose_name="No interaction (visitors)",
            ),
        ),
    ]



# Django
from django.contrib import admin
from django.utils.translation import gettext as _

# Third party
from parler.admin import TranslatableAdmin

# Local application / specific library imports
from .forms import ApplicationAdminForm, CookieAdminForm
from .models import Application, Cookie, DailyConsentStats, Purpose


class CookieInline(admin.TabularInline):
    model = Cookie
    form = CookieAdminForm


@admin.register(Application)
class ApplicationAdmin(TranslatableAdmin):
    list_display = (
        "name",
        "cookies_number",
        "purposes_list",
        "camel_case_name",
        "is_active",
        "order",
    )
    list_editable = ("is_active", "order")
    form = ApplicationAdminForm
    inlines = [CookieInline]
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "name",
                    "description",
                    "purposes",
                    "is_required",
                    "is_active",
                    "no_consent",
                    "slug",
                    "order",
                )
            },
        ),
    )

    @admin.display(description=_("Purposes"))
    def purposes_list(self, obj):
        return ", ".join([p.name for p in obj.purposes.all()])

    @admin.display(description=_("Cookies"))
    def cookies_number(self, obj):
        return obj.cookie_set.count()

    def get_prepopulated_fields(self, request, obj=None):
        # can't use `prepopulated_fields = ..` because it breaks the admin validation
        # for translated fields. This is the official django-parler workaround.
        return {"slug": ("name",)}


@admin.register(Purpose)
class PurposeAdmin(TranslatableAdmin):
    list_display = ("name", "order")
    list_editable = ("order",)


@admin.register(DailyConsentStats)
class DailyConsentStatsAdmin(admin.ModelAdmin):
    list_display = (
        "date",
        "accept_count",
        "decline_count",
        "save_count",
        "show_count",
        "no_interaction_count",
    )
    list_filter = ("date",)
    readonly_fields = (
        "date",
        "accept_count",
        "decline_count",
        "save_count",
        "show_count",
        "no_interaction_count",
    )

    def has_add_permission(self, request):
        return False



# Django
from django.core.cache import cache

VERSION_KEY = "cookie_optin_config_version"


def invalidate_config_cache():
    """Bump config cache version so all cached config.js responses are invalidated.

    The version is stored in the cache backend, not in code, so server restarts
    do not reset it (with persistent backends). With in-memory cache, the cache
    is empty after restart so we simply regenerate; no stale content is served.
    """
    try:
        cache.incr(VERSION_KEY)
    except (ValueError, TypeError):
        cache.set(VERSION_KEY, 1, timeout=None)



# Generated by Django 4.2.28 on 2026-03-03

# Django
from django.db import migrations


def recalculate_show_and_no_interaction(apps, schema_editor):
    """
    Recalculate show_count and no_interaction_count based on interaction totals.
    - show_count = 140% of (accept_count + decline_count + save_count)
    - no_interaction_count = 20% of (accept_count + decline_count + save_count)
    """
    DailyConsentStats = apps.get_model("cookie_optin", "DailyConsentStats")
    for row in DailyConsentStats.objects.all():
        total = row.accept_count + row.decline_count + row.save_count
        row.show_count = round(total * 1.4)
        row.no_interaction_count = round(total * 0.2)
        row.save()


def noop(apps, schema_editor):
    pass


class Migration(migrations.Migration):
    dependencies = [
        ("cookie_optin", "0007_add_show_and_no_interaction_counts"),
    ]

    operations = [
        migrations.RunPython(recalculate_show_and_no_interaction, noop),
    ]



# Django
from django.conf import settings
from django.contrib.sites.models import Site
from django.db import models
from django.utils.functional import cached_property
from django.utils.html import escapejs
from django.utils.translation import gettext_lazy as _
from django.utils.translation import pgettext_lazy

# Third party
from parler.models import TranslatableModel, TranslatedFields


class Purpose(TranslatableModel):
    translations = TranslatedFields(
        name=models.CharField(verbose_name=_("Name"), max_length=100)
    )

    order = models.IntegerField(verbose_name=_("Order"), default=0)

    class Meta:
        verbose_name = _("Purpose")
        verbose_name_plural = _("Purposes")
        ordering = ["order"]

    def __str__(self):
        return self.name


class Application(TranslatableModel):
    translations = TranslatedFields(
        name=models.CharField(verbose_name=_("Name"), max_length=100),
        description=models.CharField(verbose_name=_("Description"), max_length=500),
    )
    slug = models.SlugField(verbose_name=_("Slug"), max_length=100)
    purposes = models.ManyToManyField(Purpose, verbose_name=_("Purposes"))
    no_consent = models.BooleanField(
        default=False,
        verbose_name=pgettext_lazy("for an application", "Do not need consent"),
        help_text=_(
            "The application will load even before the user gave explicit consent. Use only if the application does not collect user data."
        ),
    )
    is_required = models.BooleanField(
        default=False,
        verbose_name=pgettext_lazy("for an application", "Is required"),
        help_text=_(
            "Only if the application is required for the site to function correctly."
        ),
    )
    is_active = models.BooleanField(
        default=False, verbose_name=pgettext_lazy("for an application", "Is active")
    )
    order = models.IntegerField(verbose_name=_("Order"), default=0)

    class Meta:
        verbose_name = _("Application")
        verbose_name_plural = _("Applications")
        ordering = ["order"]

    @cached_property
    def camel_case_name(self):
        return "".join([s.capitalize() for s in self.slug.rsplit("-")])

    camel_case_name.short_description = _("Identifier")

    def __str__(self):
        return self.name


class Cookie(models.Model):
    application = models.ForeignKey(
        Application, verbose_name=_("Application"), on_delete=models.CASCADE
    )
    name = models.CharField(verbose_name=_("Name"), max_length=250)
    is_regexp = models.BooleanField(default=False, verbose_name=_("Regular expression"))
    path = models.CharField(verbose_name=_("Path"), max_length=250, blank=True)
    site = models.ForeignKey(Site, verbose_name=_("Site"), on_delete=models.CASCADE)

    class Meta:
        verbose_name = _("Cookie")
        verbose_name_plural = _("Cookies")

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        if self.path is None or self.path == "":
            self.path = "/"
        super().save(*args, **kwargs)

    @cached_property
    def prepared_name(self):
        domains = [self.site.domain]
        prepared_name = ""

        # Some cookie apply to domain without 'www' so we need to test all
        if self.site.domain.startswith("www"):
            domains.append(self.site.domain.split("www")[-1])

        if self.is_regexp:
            name = f"/{escapejs(self.name)}/"
        else:
            name = f"'{escapejs(self.name)}'"

        if self.path is not None:
            path = escapejs(self.path)
            loop_name = "[{},'{}','{}']"
        else:
            loop_name = "[{},'{}']"

        if settings.DEBUG:
            domains = ["localhost"]

        for i, domain in enumerate(domains):
            current_loop_name = loop_name
            if i < len(domains) - 1:
                current_loop_name += ", "
            domain = escapejs(domain)
            current_loop_name = current_loop_name.format(name, path, domain)
            prepared_name += current_loop_name
        return prepared_name


class DailyConsentStats(models.Model):
    """One entry per day with counters for each choice type."""

    date = models.DateField(
        unique=True,
        verbose_name=_("Date"),
    )
    accept_count = models.PositiveIntegerField(
        default=0,
        verbose_name=_("Accepts"),
    )
    decline_count = models.PositiveIntegerField(
        default=0,
        verbose_name=_("Declines"),
    )
    save_count = models.PositiveIntegerField(
        default=0,
        verbose_name=_("Saves (custom)"),
    )
    show_count = models.PositiveIntegerField(
        default=0,
        verbose_name=_("Banner displays (visitors)"),
    )
    no_interaction_count = models.PositiveIntegerField(
        default=0,
        verbose_name=_("No interaction (visitors)"),
    )
    # JSON: {"FR": 5, "US": 12} - country code (ISO 3166-1 alpha-2) -> visitor count
    country_counts = models.JSONField(
        default=dict,
        blank=True,
        verbose_name=_("Visitors by country"),
    )
    # JSON: {"organic": 42, "direct": 18, "paid": 5, "social": 8, "referral": 12}
    source_counts = models.JSONField(
        default=dict,
        blank=True,
        verbose_name=_("Visitors by traffic source"),
    )
    # JSON: {"google.com": 25, "facebook.com": 10} - referrer domain -> visitor count
    referral_counts = models.JSONField(
        default=dict,
        blank=True,
        verbose_name=_("Visitors by referral domain"),
    )

    class Meta:
        verbose_name = _("Daily consent stats")
        verbose_name_plural = _("Daily consent stats")
        ordering = ["-date"]

    def __str__(self):
        return str(self.date)

    @property
    def total(self):
        return self.accept_count + self.decline_count + self.save_count

    @property
    def acceptance_rate(self):
        total = self.total
        return (
            round(100 * (self.accept_count + self.save_count) / total) if total else 0
        )



# Django
from django import forms

# Third party
from parler.forms import TranslatableModelForm

# Local application / specific library imports
from .models import Application, Cookie


class CookieAdminForm(forms.ModelForm):
    class Meta:
        model = Cookie
        fields = ["application", "name", "is_regexp", "path", "site"]
        widgets = {"path": forms.TextInput(attrs={"placeholder": "/"})}


class ApplicationAdminForm(TranslatableModelForm):
    class Meta:
        model = Application
        exclude = []
        widgets = {"slug": forms.HiddenInput}



# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("cookie_optin", "0001_initial")]

    operations = [
        migrations.AddField(
            model_name="application",
            name="is_active",
            field=models.BooleanField(default=False, verbose_name="Is active"),
        )
    ]



# Django
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

PRIVACY_URL = getattr(settings, "COOKIE_OPTIN_PRIVACY_URL", None)
API_PATH = getattr(settings, "COOKIE_OPTIN_API_PATH", "/cookie-optin/api/")
ENABLE_OVERVIEW = getattr(settings, "COOKIE_OPTIN_ENABLE_OVERVIEW", False)
CONFIG_CACHE = getattr(settings, "COOKIE_OPTIN_CONFIG_CACHE", False)
CONFIG_CACHE_TIMEOUT = getattr(settings, "COOKIE_OPTIN_CONFIG_CACHE_TIMEOUT", 3600)
# localStorage key for theme sync with CMS toolbar (light/dark).
# Must match django-cms / djangocms-admin-style key: "theme"
THEME_STORAGE_KEY = getattr(settings, "COOKIE_OPTIN_THEME_STORAGE_KEY", "theme")

if PRIVACY_URL is None:
    raise ImproperlyConfigured("COOKIE_OPTIN_PRIVACY_URL must be defined")



# Generated by Django 5.2.11 on 2026-02-23

# Django
from django.db import migrations, models


def fill_path_empty(apps, schema_editor):
    Cookie = apps.get_model("cookie_optin", "Cookie")
    Cookie.objects.filter(path__isnull=True).update(path="")


class Migration(migrations.Migration):

    dependencies = [
        ("cookie_optin", "0004_alter_application_options_alter_purpose_options_and_more"),
    ]

    operations = [
        migrations.RunPython(fill_path_empty, migrations.RunPython.noop),
        migrations.AlterField(
            model_name="cookie",
            name="path",
            field=models.CharField(
                blank=True,
                max_length=250,
                null=False,
                verbose_name="Path",
            ),
        ),
    ]
